;; -*- no-byte-compile: t; lexical-binding: nil -*-
(define-package "advance-words-count" "0.8.8"
  "A Package Provides Extended `count-words'."
  '((pos-tip "0"))
  :url "https://github.com/LdBeth/advance-words-count.el/"
  :commit "ff4b43678548f9a647bf3017bf78e55fc260913f"
  :revdesc "ff4b43678548"
  :keywords '("wp")
  :authors '(("LdBeth" . "andpuke@foxmail.com"))
  :maintainers '(("LdBeth" . "andpuke@foxmail.com")))
